@@ -13,7 +13,7 @@ import importlib_metadata
 
 from .error import SeedError, SchemaError, DataError, DatabaseError, PathError
 from .model import SimpleEnumMeta, SimpleEnum, Format, Comparator, DATETIME_FORMAT, ModelJSONEncoder, \
-    ModelField, BaseModel, relation
+    ModelField, RelationField, BaseModel, relation
 from .admin import register, registered_models
 from .mongosupport import MongoModel, connect, populate_model, populate_search
 
