@@ -78,16 +78,32 @@ def _prepare_jinja2_env():
         args = request.args.copy()
         for key, value in new_values.items():
             args[key] = value
-        return '{}?{}'.format(request.path, url_encode(args))
+        return url_encode(args)
 
     def new_model(class_name):
         """ New a model by class name. """
         klass = globals()[class_name]
         return klass()
 
+    def match_field(fields, matcher):
+        """ Get the first matching field from columns.
+
+        :param fields - list of field name
+        :param matcher - name|title|\w+_name
+        """
+        matcher = re.compile(matcher if matcher.startswith('(') else f'({matcher})')
+        if isinstance(fields, dict):
+            fields = fields.keys()
+        #
+        for f in fields:
+            if matcher.match(f):
+                return f
+        # If no matching, return nothing
+        return None
+
     env.globals['update_query'] = update_query
     env.globals['new_model'] = new_model
-
+    env.globals['match_field'] = match_field
     #
     return env
 
@@ -101,13 +117,16 @@ def _gen(s: str):
         logger.error(f'Can not find seed file {seed_file}')
         return False
     # Out dir should be same with seed
-    out_dir = s
-    if not os.path.exists(out_dir):
-        os.mkdir(out_dir)
+    # e.g,
+    #   app.sd -> app
+    #   miniapp.sd -> miniapp
+    output_path = s
+    if not os.path.exists(output_path):
+        os.mkdir(output_path)
     #
     # Parse seed file, which is in yaml format
     # e.g,
-    # template: kapok
+    # template: apple
     # blueprints:
     #   - name: dashboard
     #     views:
@@ -124,20 +143,20 @@ def _gen(s: str):
             logger.error(f'Can not parse seed file {seed_file}, {e}')
             return False
     #
-    # Get template name from seed file, and then use .name as the template folder
-    # e.g, kapok -> .kapok
+    # Get template name from seed file, and then use .name/seed name as the template folder, as one template can support many platforms
+    # e.g, app.sd using template apple -> .apple/app/
     #
-    template = '.' + seed_content['template']
-    if not os.path.exists(seed_file):
+    template_path = os.path.join('.' + seed_content['template'], s)
+    if not os.path.exists(template_path):
         # TODO: Download template to the folder
-        logger.error(f'Can not find template folder {template}')
+        logger.error(f'Can not find template folder {template_path}')
         return False
     #
     # Import models package
     # 1. Find all the models definition in models package, please import all models in __init__.py
     #
-    module_name = os.path.basename(out_dir)
-    module_spec = importlib.util.spec_from_file_location(module_name, os.path.join(out_dir, '__init__.py'))
+    module_name = os.path.basename(output_path)
+    module_spec = importlib.util.spec_from_file_location(module_name, os.path.join(output_path, '__init__.py'))
     module = importlib.util.module_from_spec(module_spec)
     sys.modules[module_name] = module
     module_spec.loader.exec_module(module)
@@ -192,12 +211,10 @@ def _gen(s: str):
     # Do generation logic recursively
     #
     env = _prepare_jinja2_env()
-    tempate_path = template
-    output_path = out_dir
-    logger.info(f'Generate template {template}: {tempate_path} -> {output_path}')
+    logger.info(f'Generate template {template_path} -> {output_path}')
     # Use depth-first to copy templates to output path, converting all the names and render in the meanwhile
-    for d in os.listdir(tempate_path):
-        _recursive_render(tempate_path, output_path, d, context, env)
+    for d in os.listdir(template_path):
+        _recursive_render(template_path, output_path, d, context, env)
 
 
 def _recursive_render(t_base, o_base, name, context, env):
